@@ -59,9 +59,11 @@
 #include <private/qquicksprite_p.h>
 #include <QAbstractAnimation>
 #include <QtQml/qqml.h>
-#include <private/qv8engine_p.h> //For QQmlV4Handle
 #include <private/qv4util_p.h>
+#include <private/qv4global_p.h>
+#include <private/qv4staticvalue_p.h>
 #include "qtquickparticlesglobal_p.h"
+#include "qquickparticleflatset_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -116,6 +118,8 @@ public:
 
     int top();
 
+    bool isEmpty() const { return m_end == 0; }
+
     QSet<QQuickParticleData*> pop();
 
     void clear();
@@ -203,6 +207,8 @@ public:
 
     QString name();
 
+    bool isActive() { return freeList.count() > 0; }
+
     void setSize(int newSize);
 
     const ID index;
@@ -335,7 +341,7 @@ public:
 
     float curSize(QQuickParticleSystem *particleSystem) const;
     void clone(const QQuickParticleData& other);//Not =, leaves meta-data like index
-    QQmlV4Handle v4Value(QQuickParticleSystem *particleSystem);
+    QV4::ReturnedValue v4Value(QQuickParticleSystem *particleSystem);
     void extendLife(float time, QQuickParticleSystem *particleSystem);
 
     static inline Q_DECL_CONSTEXPR float EPSILON() Q_DECL_NOTHROW { return 0.001f; }
@@ -350,6 +356,7 @@ class Q_QUICKPARTICLES_PRIVATE_EXPORT QQuickParticleSystem : public QQuickItem
     Q_PROPERTY(bool running READ isRunning WRITE setRunning NOTIFY runningChanged)
     Q_PROPERTY(bool paused READ isPaused WRITE setPaused NOTIFY pausedChanged)
     Q_PROPERTY(bool empty READ isEmpty NOTIFY emptyChanged)
+    QML_NAMED_ELEMENT(ParticleSystem)
 
 public:
     explicit QQuickParticleSystem(QQuickItem *parent = nullptr);
@@ -407,7 +414,7 @@ public:
     int systemSync(QQuickParticlePainter* p);
 
     //Data members here for ease of related class and auto-test usage. Not "public" API. TODO: d_ptrize
-    QSet<QQuickParticleData*> needsReset;
+    QtQuickParticlesPrivate::QFlatSet<QQuickParticleData*> needsReset;
     QVector<QQuickParticleData*> bySysIdx; //Another reference to the data (data owned by group), but by sysIdx
     QQuickStochasticEngine* stateEngine;
 
