@@ -3,8 +3,8 @@
 
 /*---
 esid: sec-array.isarray
-es5id: 15.4.3.2-1-7
-description: Array.isArray applied to Function object
+es5id: 15.4.3.2-1-10
+description: Array.isArray applied to RegExp object
 ---*/
 
-assert.sameValue(Array.isArray(function() {}), false, 'Array.isArray(function () { })');
+assert.sameValue(Array.isArray(new RegExp()), false, 'Array.isArray(new RegExp())');
