@@ -3,10 +3,8 @@
 
 /*---
 esid: sec-array.prototype.every
-es5id: 15.4.4.16-8-11
-description: >
-    Array.prototype.every returns true when all calls to callbackfn
-    return true
+es5id: 15.4.4.16-8-13
+description: Array.prototype.every doesn't visit expandos
 ---*/
 
 var callCnt = 0;
@@ -18,6 +16,8 @@ function callbackfn(val, idx, obj)
 }
 
 var arr = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9];
+arr["i"] = 10;
+arr[true] = 11;
 
 
 assert.sameValue(arr.every(callbackfn), true, 'arr.every(callbackfn)');
