@@ -3,38 +3,27 @@
 
 /*---
 esid: sec-array.prototype.reduceright
-es5id: 15.4.4.22-8-b-iii-1-30
+es5id: 15.4.4.22-8-b-iii-1-12
 description: >
-    Array.prototype.reduceRight - element changed by getter on current
-    iteration is observed in subsequent iterations on an Array
+    Array.prototype.reduceRight - element to be retrieved is own
+    accessor property that overrides an inherited data property on an
+    Array
 ---*/
 
 var testResult = false;
 
 function callbackfn(prevVal, curVal, idx, obj) {
   if (idx === 1) {
-    testResult = (curVal === 1 && prevVal === 2);
+    testResult = (prevVal === "20");
   }
 }
 
-var arr = [0];
-var preIterVisible = false;
-
-Object.defineProperty(arr, "1", {
-  get: function() {
-    if (preIterVisible) {
-      return 1;
-    } else {
-      return "20";
-    }
-  },
-  configurable: true
-});
+Array.prototype[2] = 2;
+var arr = [0, 1];
 
 Object.defineProperty(arr, "2", {
   get: function() {
-    preIterVisible = true;
-    return 2;
+    return "20";
   },
   configurable: true
 });
