@@ -2,20 +2,17 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-15gs
+es5id: 15.3.5.4_2-13gs
 description: >
-    Strict mode - checking access to strict function caller from
-    strict function (New'ed object from FunctionDeclaration defined
-    within strict mode)
+    Strict mode - checking access to non-strict function caller from
+    strict function (indirect eval used within strict mode)
 flags: [onlyStrict]
 ---*/
 
-function f() {
-  gNonStrict();
-}
+var my_eval = eval;
 
 assert.throws(TypeError, function() {
-  new f();
+  my_eval("gNonStrict();");
 });
 
 function gNonStrict() {
