@@ -2,15 +2,15 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.3-4-32
+es5id: 15.2.3.3-4-60
 description: >
     Object.getOwnPropertyDescriptor returns data desc for functions on
-    built-ins (Object.prototype.propertyIsEnumerable)
+    built-ins (Array.prototype.reduceRight)
 ---*/
 
-var desc = Object.getOwnPropertyDescriptor(Object.prototype, "propertyIsEnumerable");
+var desc = Object.getOwnPropertyDescriptor(Array.prototype, "reduceRight");
 
-assert.sameValue(desc.value, Object.prototype.propertyIsEnumerable, 'desc.value');
+assert.sameValue(desc.value, Array.prototype.reduceRight, 'desc.value');
 assert.sameValue(desc.writable, true, 'desc.writable');
 assert.sameValue(desc.enumerable, false, 'desc.enumerable');
 assert.sameValue(desc.configurable, true, 'desc.configurable');
