@@ -2,19 +2,16 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.8-2-a-4
-description: Object.seal - 'P' is own accessor property
+es5id: 15.2.3.8-2-a-11
+description: >
+    Object.seal - 'P' is own property of a Number object that uses
+    Object's [[GetOwnProperty]]
 includes: [propertyHelper.js]
 ---*/
 
-var obj = {};
+var obj = new Number(-1);
 
-Object.defineProperty(obj, "foo", {
-  get: function() {
-    return 10;
-  },
-  configurable: true
-});
+obj.foo = 10;
 
 assert(Object.isExtensible(obj));
 Object.seal(obj);
