@@ -2,23 +2,20 @@
 // See LICENSE for details.
 
 /*---
-es6id: S25.4.4.3_A6.2_T1
+es6id: S25.4.4.3_A6.1_T1
 author: Sam Mikes
-description: Promise.race([p1]) settles immediately
+description: Promise.race([1]) settles immediately
 includes: [promiseHelper.js]
 flags: [async]
 ---*/
 
 var sequence = [];
 
-var p1 = Promise.reject(1),
-  p = Promise.race([p1]);
+var p = Promise.race([1]);
 
 sequence.push(1);
 
 p.then(function() {
-  $ERROR("Should not fulfill.");
-}, function() {
   sequence.push(4);
   checkSequence(sequence, "This happens second");
 }).catch($DONE);
