@@ -2,10 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 11.8.3-1
+es5id: 11.8.3-2
 description: >
     11.8.3 Less-than-or-equal Operator - Partial left to right order
-    enforced when using Less-than-or-equal operator: valueOf <= valueOf
+    enforced when using Less-than-or-equal operator: valueOf <=
+    toString
 ---*/
 
         var accessed = false;
@@ -16,7 +17,7 @@ description: >
             }
         };
         var obj2 = {
-            valueOf: function () {
+            toString: function () {
                 if (accessed === true) {
                     return 4;
                 } else {
