@@ -10,17 +10,27 @@ info: |
     local functions must shadow outer functions
     local function expresssions must shadow outer function expressions
     eval should use the appended object to the scope chain
-es5id: 12.14-3
-description: >
-    catch doesn't change declaration scope - var declaration are
-    visible outside when name different from catch parameter
+es5id: 12.14-7
+description: catch introduces scope - scope removed when exiting catch block
 ---*/
 
-  try {
-    throw new Error();
-  }
-  catch (e) {
-    var foo = "declaration in catch";
-  }
+      var o = {foo: 1};
+      var catchAccessed = false;
+      
+      try {
+        throw o;
+      }
+      catch (expObj) {
+        catchAccessed = (expObj.foo == 1);
+      }
+      assert(catchAccessed, '(expObj.foo == 1)');
 
-assert.sameValue(foo, "declaration in catch");
+
+      catchAccessed = false;
+      try {
+        expObj;
+      }
+      catch (e) {
+        catchAccessed = e instanceof ReferenceError
+      }
+      assert(catchAccessed, 'e instanceof ReferenceError');
