@@ -2,14 +2,13 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.2.1-27-s
+es5id: 12.2.1-22-s
 description: >
-    eval as local var identifier assigned to throws SyntaxError in
-    strict mode
-flags: [onlyStrict]
+    arguments as global var identifier throws SyntaxError in strict
+    mode
 ---*/
 
-
+    var indirectEval = eval;
 assert.throws(SyntaxError, function() {
-    eval('function foo() { var eval = 42, a;}');
+	    indirectEval("'use strict'; var arguments;");
 });
