@@ -5,9 +5,9 @@
 info: |
     No matter how control leaves the embedded 'Statement',
     the scope chain is always restored to its former state
-es5id: 12.10_A3.5_T3
+es5id: 12.10_A3.4_T3
 description: >
-    Using "with" statement within "for-in" statement, leading to
+    Using "with" statement within iteration statement, leading to
     completion by exception
 flags: [noStrict]
 ---*/
@@ -23,12 +23,12 @@ var myObj = {
 }
 
 try {
-    for(var prop in myObj){
+    do{
         with(myObj){
             throw value;
             p1 = 'x1';
         }
-    }
+    } while(false);
 } catch(e){
     result = p1;
 }
