@@ -36,34 +36,35 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.8.1.1-2.js';
+gTestfile = '15.8.1.2-2.js';
 
 /**
-   File Name:          15.8.1.1-2.js
-   ECMA Section:       15.8.1.1.js
+   File Name:          15.8.1.2-1.js
+   ECMA Section:       15.8.2.js
    Description:        All value properties of the Math object should have
    the attributes [DontEnum, DontDelete, ReadOnly]
 
-   this test checks the DontDelete attribute of Math.E
+   this test checks the DontDelete attribute of Math.LN10
 
    Author:             christine@netscape.com
    Date:               16 september 1997
 */
-var SECTION = "15.8.1.1-2";
+
+var SECTION = "15.8.1.2-2";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Math.E";
+var TITLE   = "Math.LN10";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-var MATH_E = 2.7182818284590452354
-  new TestCase( SECTION,
-		"delete(Math.E)",               
-		false,   
-		eval("delete Math.E") );
 new TestCase( SECTION,
-	      "delete(Math.E); Math.E",       
-	      MATH_E,  
-	      eval("delete Math.E; Math.E") );
+	      "delete( Math.LN10 ); Math.LN10",  
+	      2.302585092994046,      
+	      eval("delete(Math.LN10); Math.LN10") );
+
+new TestCase( SECTION,
+	      "delete( Math.LN10 ); ",            
+	      false,                 
+	      eval("delete(Math.LN10)") );
 
 test();
