@@ -36,11 +36,11 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.5.4.12-4.js';
+gTestfile = '15.5.4.11-5.js';
 
 /**
-   File Name:          15.5.4.12-1.js
-   ECMA Section:       15.5.4.12 String.prototype.toUpperCase()
+   File Name:          15.5.4.11-5.js
+   ECMA Section:       15.5.4.11 String.prototype.toLowerCase()
    Description:
 
    Returns a string equal in length to the length of the result of converting
@@ -51,7 +51,7 @@ gTestfile = '15.5.4.12-4.js';
    case the uppercase equivalent is used instead. (The canonical Unicode 2.0 case
    mapping shall be used, which does not depend on implementation or locale.)
 
-   Note that the toUpperCase function is intentionally generic; it does not require
+   Note that the toLowerCase function is intentionally generic; it does not require
    that its this value be a String object. Therefore it can be transferred to other
    kinds of objects for use as a method.
 
@@ -59,28 +59,33 @@ gTestfile = '15.5.4.12-4.js';
    Date:               12 november 1997
 */
 
-var SECTION = "15.5.4.12-1";
+var SECTION = "15.5.4.11-5";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "String.prototype.toUpperCase()";
+var TITLE   = "String.prototype.toLowerCase()";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
+new TestCase( SECTION,  "String.prototype.toLowerCase.length",        0,          String.prototype.toLowerCase.length );
+
+new TestCase( SECTION,  "delete String.prototype.toLowerCase.length", false,      delete String.prototype.toLowerCase.length );
+
+new TestCase( SECTION,  "delete String.prototype.toLowerCase.length; String.prototype.toLowerCase.length", 0,      eval("delete String.prototype.toLowerCase.length; String.prototype.toLowerCase.length") );
+
 // Cyrillic (part)
 // Range: U+0400 to U+04FF
 for ( var i = 0x0400; i <= 0x047F; i++ ) {
-  var U =new Unicode( i );
+  var U = new Unicode( i );
 /*
   new TestCase(   SECTION,
-  "var s = new String( String.fromCharCode("+i+") ); s.toUpperCase()",
-  U.upper,
-  eval("var s = new String( String.fromCharCode("+i+") ); s.toUpperCase()") );
+  "var s = new String( String.fromCharCode("+i+") ); s.toLowerCase()",
+  String.fromCharCode(U.lower),
+  eval("var s = new String( String.fromCharCode("+i+") ); s.toLowerCase()") );
 */
   new TestCase(   SECTION,
-		  "var s = new String( String.fromCharCode("+i+") ); s.toUpperCase().charCodeAt(0)",
-		  U.upper,
-		  eval("var s = new String( String.fromCharCode(i) ); s.toUpperCase().charCodeAt(0)") );
-
+		  "var s = new String( String.fromCharCode("+i+") ); s.toLowerCase().charCodeAt(0)",
+		  U.lower,
+		  eval("var s = new String( String.fromCharCode(i) ); s.toLowerCase().charCodeAt(0)") );
 }
 
 test();
